import zipfile
from zipfile import BadZipFile
import os
from io import BytesIO

from PIL import Image

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib import messages
from django.template.defaultfilters import slugify
from django.core.files.base import ContentFile

from .models import Gallery, Photo


class UploadZipForm(forms.Form):
    zip_file = forms.FileField()

    name = forms.CharField(label=_('Имя'), max_length=250, required=False)
    gallery = forms.ModelChoiceField(Gallery.objects.all(), label=_('Галерея'), required=False)
    caption = forms.CharField(label=_('Подпись'), required=False)

    def clean_zip_file(self):
        """Открываю zip-архив, чтобы убедиться, что это действительный zip-архив."""
        zip_file = self.cleaned_data['zip_file']
        try:
            zip = zipfile.ZipFile(zip_file)
        except BadZipFile as e:
            raise forms.ValidationError(str(e))
        bad_file = zip.testzip()
        if bad_file:
            zip.close()
            raise forms.ValidationError('"%s" архив .zip поврежден.' % bad_file)
        zip.close()
        return zip_file

    def clean_title(self):
        name = self.cleaned_data['name']
        if name and Gallery.objects.filter(title=name).exists():
            raise forms.ValidationError(_('Галерея с таким названием уже существует.'))
        return name

    def clean(self):
        cleaned_data = super(UploadZipForm, self).clean()
        if not self['name'].errors:
            if not cleaned_data.get('name', None) and not cleaned_data['gallery']:
                raise forms.ValidationError(
                    _('Выберите существующую галерею или введите название для новой галереи.'))
        return cleaned_data

    def save(self, request=None, zip_file=None):
        if not zip_file:
            zip_file = self.cleaned_data['zip_file']
        zip = zipfile.ZipFile(zip_file)
        count = 1
        if self.cleaned_data['gallery']:
            gallery = self.cleaned_data['gallery']
        else:
            gallery = Gallery.objects.create(name=self.cleaned_data['name'], slug=slugify(self.cleaned_data['name']))
        for filename in sorted(zip.namelist()):

            if filename.startswith('__') or filename.startswith('.'):
                continue

            if os.path.dirname(filename):
                if request:
                    messages.warning(request,
                                     _('Игнорирование файла "{filename}" который в подпапке; все изображения должны '
                                       'быть в корене zip.').format(filename=filename), fail_silently=True)
                continue

            data = zip.read(filename)

            if not len(data):
                continue

            photo_title_root = self.cleaned_data['name'] if self.cleaned_data['name'] else gallery.title

            # Возможно, фотография уже существует с тем же слагом
            # делаю петлю, пока не найдем slug, который доступен.
            while True:
                img_title = ' '.join([photo_title_root, str(count)])
                slug = slugify(img_title)
                if Photo.objects.filter(slug=slug).exists():
                    count += 1
                    continue
                break

            photo = Photo(name=img_title, slug=slug, captions=self.cleaned_data['caption'])

            # Основная проверка, что это изображение.
            try:
                file = BytesIO(data)
                opened = Image.open(file)
                opened.verify()
            except Exception:
                # Pillow не распознает изображение.
                # Если «плохой» файл найден, мы просто пропускаем его.
                # Сообщаю это пользователю.
                if request:
                    messages.warning(request,
                                     _('Не удалось обработать файл "{0}" в архиве .zip.').format(
                                         filename),
                                     fail_silently=True)
                continue

            contentfile = ContentFile(data)
            photo.image.save(filename, contentfile)
            photo.save()
            gallery.images.add(photo)
            count += 1

        zip.close()

        if request:
            messages.success(request,
                             _('Фотографии были добавлены в галерею "{0}".').format(
                                 gallery.name),
                             fail_silently=True)


# Generated by Django 2.2.3 on 2019-07-14 07:29

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Photo',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250, verbose_name='Название')),
                ('image', models.ImageField(upload_to='gallery', verbose_name='Изображение')),
                ('captions', models.CharField(blank=True, max_length=250, verbose_name='Подпись')),
                ('create_date', models.DateTimeField(auto_now_add=True, verbose_name='Дата добавления')),
                ('slug', models.SlugField(max_length=255)),
            ],
            options={
                'verbose_name': 'Иллюстрация',
                'verbose_name_plural': 'Иллюстрации',
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='Gallery',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=250, verbose_name='Название')),
                ('captions', models.TextField(blank=True, max_length=250, verbose_name='Описание')),
                ('create_date', models.DateTimeField(auto_now_add=True, verbose_name='Дата добавления')),
                ('slug', models.SlugField(max_length=255)),
                ('images', models.ManyToManyField(to='oms_gallery.Photo', verbose_name='Изображение')),
            ],
            options={
                'verbose_name': 'Галерея',
                'verbose_name_plural': 'Галереи',
                'ordering': ['name'],
            },
        ),
    ]



# Generated by Django 3.0.3 on 2020-02-29 08:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('oms_gallery', '0002_auto_20190714_1348'),
    ]

    operations = [
        migrations.AlterField(
            model_name='photo',
            name='captions',
            field=models.TextField(blank=True, verbose_name='Подпись'),
        ),
    ]



from django.urls import reverse
from django.utils.translation import ugettext_lazy as _
from django.db import models
from PIL import Image
import os

from django.utils.safestring import mark_safe


class Photo(models.Model):
    """Класс модели галереи"""
    name = models.CharField(_("Название"), max_length=250)
    image = models.ImageField(_("Изображение"), upload_to='gallery')
    captions = models.TextField(_("Подпись"), blank=True)
    сopyright = models.CharField(_("Автор"), max_length=250, blank=True, null=True)
    create_date = models.DateTimeField(_("Дата добавления"), auto_now_add=True)
    slug = models.SlugField(max_length=255)

    def __str__(self):
        return self.name

    # Добавляем к свойствам объектов модели путь к миниатюре
    def _get_mini_path(self):
        return self._add_mini(self.image.path)

    mini_path = property(_get_mini_path)

    # Добавляем к свойствам объектов модели урл миниатюры
    def _get_mini_url(self):
        return self._add_mini(self.image.url)

    mini_url = property(_get_mini_url)

    # Добавляем к свойствам объектов модели html код для отображения миниатюры
    # Сделано в модели для упрощения вывода в админке. Смотрим далее.
    def get_mini_html(self):
        return mark_safe('<a class="image-picker" href="{}"><img src="{}" alt="{}"/></a>'.format(
            self.image.url, self._add_mini(self.image.url), self.captions
        ))

    mini_html = property(get_mini_html)
    get_mini_html.short_description = _('Изображение')
    get_mini_html.allow_tags = True

    # def to_internal_value(self, value):
    #     f = value.split("/").pop().split(".").pop(1)
    #     if f == "jpeg" or f == "jpg" or f == "webp":
    #         way = "tmp/img{}.j2p".format(value.split("/").pop().split(".").pop(0))
    #         outputPath = os.path.join(settings.MEDIA_ROOT, way)
    #         # quality = 50
    #         try:
    #             Image.open(settings.MEDIA_ROOT + "/" + way)
    #         except:
    #             im = Image.open(settings.BASE_DIR + value[value.rfind('/media'):])
    #             im.save(outputPath, 'JPEG', optimize=True, quality=60)
    #         path = settings.MEDIA_URL[:settings.MEDIA_URL.find('media')] + outputPath[outputPath.rfind('media'):]
    #         return path
    #     else:
    #         return value

    # Создаем свою save
    # Добавляем:
    # - создание миниатюры
    # - удаление миниатюры и основного изображения
    #   при попытке записи поверх существующей записи
    def save(self, **kwargs):
        try:
            obj = Photo.objects.get(id=self.id)
            if obj.image.path != self.image.path:
                self._del_mini(obj.image.path)
                obj.image.delete()
        except:
            pass
        super(Photo, self).save(**kwargs)
        img = Image.open(self.image.path)
        img.thumbnail(
            (128, 128),
            Image.ANTIALIAS
        )

        f = self.image.name.split(".").pop(1)
        if f == "jpeg" or f == "jpg":
            img.save(self.mini_path, "JPEG")
        else:
            img.save(self.mini_path, f)

    # Делаем свою delete с учетом миниатюры
    def delete(self, using=None):
        try:
            obj = Photo.objects.get(id=self.id)
            self._del_mini(obj.image.path)
            obj.image.delete()
        except (Photo.DoesNotExist, ValueError):
            pass
        super(Photo, self).delete()

    def _add_mini(self, s):
        """Изменение (filename, URL) вставкой '.mini' и изменение расширения на jpg"""
        parts = s.split(".")
        parts.insert(-1, "mini")
        if parts[-1].lower() not in ['jpeg', 'jpg']:
            parts[-1] = 'jpg'
        return ".".join(parts)

    def _del_mini(self, p):
        """Удаление миниатюры с физического носителя"""
        mini_path = self._add_mini(p)
        if os.path.exists(mini_path):
            os.remove(mini_path)

    def get_absolute_url(self):
        return reverse('photo_detail', kwargs={'object_id': self.id})

    class Meta:
        verbose_name = _('Изображение')
        verbose_name_plural = _('Изображения')
        ordering = ['name']


class Gallery(models.Model):
    """Модель галереи"""
    name = models.CharField(_("Название"), max_length=250)
    images = models.ManyToManyField(Photo, verbose_name=_("Изображение"))
    captions = models.TextField(_("Описание"), max_length=250, blank=True)
    create_date = models.DateTimeField(_("Дата добавления"), auto_now_add=True)
    slug = models.SlugField(max_length=255)

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = _('Галерея')
        verbose_name_plural = _('Галереи')
        ordering = ['name']



from django.urls import path
from django.contrib import admin
from django import forms
from django.contrib.admin import helpers
from django.http import HttpResponseRedirect
from django.shortcuts import render

from .models import Photo, Gallery
from .forms import UploadZipForm


@admin.register(Photo)
class PhotoAdmin(admin.ModelAdmin):
    """Админ фото"""
    list_display = ('name', 'captions', 'get_mini_html', 'create_date', 'id')
    search_fields = ('name', )
    # admin.site.disable_action('delete_selected')
    readonly_fields = ["get_mini_html"]
    list_filter = ('create_date', )
    prepopulated_fields = {'slug': ('name',)}

    fieldsets = (
        (None, {
            'fields': ('name',),
        }),
        (None, {
            'fields': ('image',),
        }),
        (None, {
            'fields': ('captions',),
        }),
        (None, {
            'fields': ('сopyright',),
        }),
        (None, {
            'fields': ('slug',),
        }),
    )

    def get_actions(self, request):
        actions = super().get_actions(request)
        if 'delete_selected' in actions:
            del actions['delete_selected']
        return actions

    def full_delete_selected(self, request, obj):
        for o in obj.all():
            o.delete()
    full_delete_selected.short_description = 'Удалить выбранные иллюстрации'
    actions = ['full_delete_selected']

    def get_urls(self):
        urls = super().get_urls()
        my_urls = [
            path('immortal/', self.upload_zip, name="oms_upload_zip"),
        ]
        return my_urls + urls
    change_list_template = "oms_gallery/admin/change_list.html"

    def upload_zip(self, request):

        context = {
            'title': 'Загрузка изображений zip-архивом',
            'app_label': self.model._meta.app_label,
            'opts': self.model._meta,
            'has_change_permission': self.has_change_permission(request)
        }

        # Handle form request
        if request.method == 'POST':
            form = UploadZipForm(request.POST, request.FILES)
            if form.is_valid():
                form.save(request=request)
                return HttpResponseRedirect('..')
        else:
            form = UploadZipForm()
        context['form'] = form
        context['adminform'] = helpers.AdminForm(form, list([(None, {'fields': form.base_fields})]), {})
        return render(request, 'oms_gallery/admin/upload_zip.html', context)


class PhotoChoiceField(forms.ModelMultipleChoiceField):
    """Виджет checkbox с отображением миниатюр"""
    def label_from_instance(self, obj):
        return obj.get_mini_html()


class GalleryForm(forms.ModelForm):
    """Форма для галереи с виджетом"""
    images = PhotoChoiceField(widget=forms.CheckboxSelectMultiple, queryset=Photo.objects.all(), required=True)

    class Meta:
        model = Gallery
        fields = ('__all__')


@admin.register(Gallery)
class GalleryAdmin(admin.ModelAdmin):
    """Админ галереи"""
    list_display = ('name', 'captions', 'create_date', 'id')
    list_filter = ('create_date',)
    prepopulated_fields = {'slug': ('name',)}
    form = GalleryForm








from django import template

from oms_gallery.models import Gallery, Photo

register = template.Library()


@register.simple_tag()
def for_gallery(name=None, pk=None):
    """Вывод галереи по имени или id"""
    if name is not None:
        gallery = Gallery.objects.filter(name=name)
    elif pk is not None:
        gallery = Gallery.objects.filter(id=pk)
    else:
        gallery = Gallery.objects.all()
    return gallery


@register.simple_tag()
def get_photo(name=None, pk=None):
    """Вывод фото по имени или id"""
    if name is not None:
        photo = Photo.objects.filter(name=name).first()
    elif pk is not None:
        photo = Photo.objects.filter(id=pk).first()
    else:
        photo = None
    return photo


@register.simple_tag()
def for_photo(gallery=None):
    """Вывод фото имени галереи"""
    if gallery is not None:
        photos = Photo.objects.filter(gallery__name=gallery)
    else:
        photos = Photo.objects.all()
    return photos



# Generated by Django 2.2.3 on 2019-07-14 10:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('oms_gallery', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='photo',
            options={'ordering': ['name'], 'verbose_name': 'Изображение', 'verbose_name_plural': 'Изображения'},
        ),
    ]



# /usr/bin/env python
import os

from setuptools import setup

import oms_gallery


def read(fname):
    with open(os.path.join(os.path.dirname(__file__), fname)) as f:
        return f.read()


with open('requirements.txt') as f:
    reqs = f.read().splitlines()


setup(
    name="oms_gallery",
    version=oms_gallery.__version__,
    description="Image management for Django.",
    author="Omelchenko Michael - DJWOMS",
    author_email="djwoms@gmail.com",
    url="https://github.com/DJWOMS/oms-gallery",
    long_description=read('README.md'),
    packages=['oms_gallery'],
    include_package_data=True,
    zip_safe=False,
    classifiers=['Development Status :: 4 - Beta',
                 'Environment :: Web Environment',
                 'Framework :: Django',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: BSD License',
                 'Operating System :: OS Independent',
                 'Programming Language :: Python',
                 'Programming Language :: Python :: 3',
                 'Programming Language :: Python :: 3.7',
                 'Programming Language :: Python :: 3.8',
                 'Topic :: Utilities'],
    install_requires=reqs,
)
